<?php
/**
 * @file
 * Admin page(s).
 */

function robotagger_settings_form($form_state) {
  _robotagger_add_js();
  $form = array();
  $apikey = variable_get('robotaggerapi_server_apikey', '');
  if (empty($apikey)) {
    drupal_set_message(t('Before you can continue, set the api key in the !link settings.', array('!link' => l('RoboTagger-Api', 'admin/settings/robotagger_api'))), 'warning');
    return $form;
  }
  $sendrequest_description = t('Automatic - After submitting the content, the module requests the RoboTagger-Webservice. <br />Manual - You must send manually your content to the webservice on a seperate page.');
  $confirmtags_description = t('Automatic - The module saves the suggestions as taxonomy term immediately.<br />Manual - Before the suggestions are saved, you can confirm them on seperated page.');
  $form['robotagger_global_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#description' => t('Please note. You need to assign the content types for each vocabularies per hand. Or enable the checkbox %checkboxlabel in this form.', array('%checkboxlabel' => t('Un-/Assign content types to the vocabularies automatically?'))),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );
  $form['robotagger_global_wrapper']['robotagger_global_sendrequest'] = array(
    '#type' => 'select',
    '#title' => t('Send request'),
    '#options' => array('disabled' => t('Disabled'), 'auto' => t('Automatic'), 'manual' => t('Manual')),
    '#default_value' => variable_get('robotagger_global_sendrequest', 'auto'),
    '#description' => $sendrequest_description,
  );
  $form['robotagger_global_wrapper']['robotagger_global_confirmtags'] = array(
    '#type' => 'select',
    '#title' => t('Confirm returned suggestions'),
    '#options' => array('auto' => t('Automatic'), 'manual' => t('Manual')),
    '#default_value' => variable_get('robotagger_global_confirmtags', 'auto'),
    '#description' => $confirmtags_description,
  );
  $form['robotagger_global_wrapper']['robotagger_global_caching'] = array(
    '#type' => 'select',
    '#title' => t('Caching returned data'),
    '#options' => array(TRUE => t('yes'), FALSE => t('no')),
    '#default_value' => variable_get('robotagger_global_caching', TRUE),
    '#description' => t('When enabled, the RoboTagger-API caches the returned data by the RoboTagger-Webservice.'),
  );
  $form['robotagger_global_wrapper']['robotagger_global_logging'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable logging'),
    '#default_value' => variable_get('robotagger_global_logging', 1),
    '#description' => t('When checked, the RoboTagger module logs all requests to the RoboTagger-Webservice.'),
  );
  $nodetypes = node_get_types('names');
  $form['robotagger_global_wrapper']['robotagger_global_nodetypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Supported content types'),
    '#options' => $nodetypes,
    '#default_value' => variable_get('robotagger_global_nodetypes', array()),
    '#description' => t('Choose the content types for the RoboTagger-Webservice.'),
  );
  $form['robotagger_global_wrapper']['robotagger_global_non_cck_fields'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Non-CCK text fields'),
    '#options' => array('title' => t('Title field'), 'body' => t('Body field')),
    '#default_value' => variable_get('robotagger_global_non_cck_fields', array('title', 'body')),
    '#description' => t('When enabled, the title/body will also be used for the RoboTagger-Webservice. But the title will be only used if at least one another text field is checked.'),
  );
  $supported_fields = array();
  if (module_exists('content')) {
    $fields = content_fields();
    foreach ($fields as $field) {
      if ($field['module'] == 'text' && $field['active']) {
        $supported_fields[$field['field_name']] = $field['field_name'];
      }
    }
    $form['robotagger_global_wrapper']['robotagger_global_cck_fields'] = array(
      '#type' => empty($supported_fields) ? 'item' : 'checkboxes',
      '#title' => t('Fields'),
      '#options' => $supported_fields,
      '#default_value' => variable_get('robotagger_global_cck_fields', array()),
      '#value' => t('No supported fields from the text module are available.'),
      '#description' => t('Which fields should be used for the RoboTagger webservice?'),
    );
  }
  $form['robotagger_global_wrapper']['robotagger_assignment_vocs'] = array(
    '#type' => 'checkbox',
    '#title' => t('Un-/Assign content types to the vocabularies automatically?'),
    '#description' => t('After submitting the form, the module assigns the checked content types to the checked vocabularies. Or remove all assignments from all vocabularies for the unchecked content types.'),
  );
  $options = _robotagger_get_vocabularies('db');
  $values = variable_get('robotagger_global_vocs', array());
  $form['robotagger_global_wrapper']['robotagger_global_vocs'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Vocabularies'),
    '#options' => $options,
    '#description' => t('Which vocabularies should be used for the RoboTagger webservice?'),
    '#default_value' => $values,
  );
  $text = array_filter($values) ? t('Deselect all') : t('Select all');
  $form['robotagger_global_wrapper']['js'] = array(
    '#type' => 'item',
    '#value' => '<a href="#" class="robotagger-js-select-all">' . $text . '</a>',
  );
  $form['robotagger_nodetype_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content type settings'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );
  foreach ($nodetypes as $mname => $uiname) {
    $w_key = 'robotagger_nodetype_' . $mname . '_wrapper';
    $form['robotagger_nodetype_wrapper'][$w_key] = array(
      '#type' => 'fieldset',
      '#title' => $uiname,
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
    );
    $s_key = 'robotagger_nodetype_' . $mname . '_settings';
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_use_global'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use default settings'),
      '#description' => t('All made settings below are ignored and the global settings are used.'),
      '#default_value' => variable_get($s_key.'_use_global', 1),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_sendrequest'] = array(
      '#type' => 'select',
      '#title' => t('Send request'),
      '#options' => array('auto' => t('Automatic'), 'manual' => t('Manual')),
      '#default_value' => variable_get($s_key.'_sendrequest', 'auto'),
      '#desctiption' => $sendrequest_description,
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_confirmtags'] = array(
      '#type' => 'select',
      '#title' => t('Confirm returned suggestions'),
      '#options' => array('auto' => t('Automatic'), 'manual' => t('Manual')),
      '#default_value' => variable_get($s_key.'_confirmtags', 'auto'),
      '#description' => $confirmtags_description,
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_non_cck_fields'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Non-CCK text fields'),
      '#options' => array('title' => t('Title field'), 'body' => t('Body field')),
      '#default_value' => variable_get($s_key.'_non_cck_fields', array('title', 'body')),
      '#description' => t('When enabled, the title or body field will be also used for the RoboTagger webservice.'),
    );
    if (module_exists('content') && !empty($supported_fields)) {
      $fieldsbyct = array();
      foreach ($supported_fields as $fieldname => $field_label) {
        $field = content_fields($fieldname,  $mname);
        if (!empty($field)) {
          $fieldsbyct[$field['field_name']] = $field['widget']['label'];
        }
      }
      $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_cck_fields'] = array(
        '#type' => empty($fieldsbyct) ? 'item' : 'checkboxes',
        '#title' => t('Field instances'),
        '#options' => $fieldsbyct,
        '#default_value' => variable_get($s_key.'_cck_fields', array()),
        '#value' => t('No supported field instances from the text module are available.'),
        '#description' => t('Which field instances should be used for the RoboTagger webservice?'),
      );
    }
  }
  $form['#submit'][] = 'robotagger_settings_form_submit';
  return system_settings_form($form);
}

/**
 * Submit handler for our settings form.
 *
 * @param array $form
 * @param array $form_state
 *
 * @see robotagger_settings_form().
 */
function robotagger_settings_form_submit($form, $form_state) {
  $nodetypes = $form_state['values']['robotagger_global_nodetypes'];
  if ($form_state['values']['robotagger_assignment_vocs']) {
    $allowed_vocs = $form_state['values']['robotagger_global_vocs'];
    $vocabularies = taxonomy_get_vocabularies();
    foreach ($nodetypes as $nodetype => $allowed) {
      // When checked than created else remove all assignments.
      if ($nodetype === $allowed) {
        _robotagger_assign_vocabulary($nodetype, $allowed_vocs);
      }
      else{
        _robotagger_unassign_vocabulary($nodetype, $vocabularies);
      }
    }
  }
}
