<?php

/**
 * @file
 * Requests the RoboTagger-API and saves the annotation suggestions, detected topic to the fields or
 * saves, declines, deletes the suggestions in the database.
 */

/**
 * A wrapper function.
 *
 * @param stdClass $node
 *   The node object.
 */
function _robotagger_process_node($node) {
  if ($node->robotagger_process_all) {
    return _robotagger_process_node_nodeapi($node);
  }
  elseif (!$node->robotagger_process_all && $node->rtg_op == 'presave') {
    return _robotagger_process_node_form($node);
  }
}

function _robotagger_process_node_nodeapi($node) {
  $langcode = $node->language;
  $settings = _robotagger_get_settings($node->type);
  /* On presave we saves the annotation in th node object.
  On insert/update in our database. Therefore we need to avoid thats the function
  runs twice.
  */
  if ($node->rtg_op != 'presave' && $settings->confirmtags == 'auto') {
    return;
  }
  if ($node->rtg_op == 'presave' && $settings->confirmtags == 'manual') {
    return;
  }
  if ($settings->sendrequest == 'disabled' || !$settings->isallow) {
    return;
  }
  if ($settings->sendrequest == 'manual' && $node->rtcheck_sendrequest == TRUE) {
    return;
  }
  $vids = array_filter($settings->vids);//remove disabled vids
  if (empty($vids)) {
    return;
  }
  $nodetypes_vids = array();
  $SQL = "SELECT v.* FROM {vocabulary} v
          INNER JOIN {vocabulary_node_types} n ON v.vid = n.vid
          WHERE n.type = '%s' AND v.vid IN(" . db_placeholders($vids, 'int') . ")";
  $query = db_query($SQL, array_merge(array($node->type), $vids));
  while ($vocabulary = db_fetch_object($query)) {
    $nodetypes_vids[$vocabulary->vid] = $vocabulary->name;
  }
  $text = '';
  if ($settings->use_body) {
    $text .= ' ' . $node->body;
  }
  foreach ($settings->supported_fields as $fieldname) {
    if (isset($node->{$fieldname})) {
      foreach ($node->{$fieldname} as $fielddelta) {
        $text .= $fielddelta['value'] . ' ';
      }
    }
  }
  if (empty($text)) {
    return;
  }
  if ($settings->use_title) {
    $text = $node->title . ' ' . $text;
  }
  if (!empty($nodetypes_vids)) {
    $data = robotagger_api_call_webservice($text, $nodetypes_vids, $langcode, $settings->caching);
    if (empty($data)) {
      return;
    }
    if ($settings->logging)  {
      watchdog('robotagger', 'Robotagger processing %content.', array('%content' => truncate_utf8($node->body, 150, TRUE, TRUE)));
    }
    $terms = taxonomy_node_get_terms($node, $key = 'tid');
    $declined = _robotagger_get_declined_suggestions($node);
    $oldtids = $newsuggestions = array();
    foreach ($nodetypes_vids as $vid => $vocname) {
      if (isset($data[$vocname])) {
        foreach ($data[$vocname] as &$item) {
          if (($value = trim($item['value'])) && !empty($value)) {
            // Is the suggestion already in our declined suggestions?
            if (!isset($declined[$vid][$value])) {
              // Does taxonomy know this.
              $tid = _robotagger_check_term_is_new($value, $vid);
              if (empty($tid)) {
                $newsuggestions[$vid][$value] = $item;
              }
              elseif (!empty($tid) && !isset($terms[$tid])) {
                $oldtids[$vid][] = array('tid' => $tid);
                // on the confirmation page we also provide suggestions, which
                // already known as term
                if ($settings->confirmtags == 'manual') {
                  $newsuggestions[$vid][$value] = $item;
                }
              }
            }
          }
        }// foreach ($data[$vocname] as &$item) {
      }// if (isset($data[$vocname])) {
    }//foreach ($nodetypes_vids as $vid => $vocname) {
    // only in presave we can add term fields
    if ($node->rtg_op == 'presave' && $settings->confirmtags == 'auto') {
      return _robotagger_process_node_suggestions($node, $newsuggestions, $oldtids);
    }
    elseif ($node->rtg_op != 'presave' && $settings->confirmtags == 'manual') {
      return _robotagger_suggestions_save($node, $newsuggestions);
    }
  }// if (!empty($nodetypes_vids)) {
}

/**
* Creates taxonomy terms for new suggestions and saves the new created terms or
* already stored terms in the node object. Works only presave mode.
*
* @param stdClass $node
*   The node object.
* @param array $newsuggestions
*   An array of annotation suggestions.
* @param array $oldtids
*   An array of tids for a certain vocabulary.
*
* @return object $node
*   The altered node object.
*/
function _robotagger_process_node_suggestions($node, $newsuggestions = array(), $oldtids = array()) {
  if (!empty($newsuggestions)) {
    foreach ($newsuggestions as $vid => $items) {
      foreach ($items as $item) {
        $term = array(
         'vid' => $vid,
         'name' => $item['value'],
        );
        $tid = taxonomy_save_term($term);
        if (!empty($tid)) {
          $oldtids[$vid][] = array('tid' => $term['tid']);
        }
      }
    }
  }
  foreach ($oldtids as $vid => $terms) {
    foreach ($terms as $i => $tid) {
      $node->taxonomy[] = taxonomy_get_term($tid['tid']);
    }
  }
  return $node;
}

/**
 * Checks if a annotation suggestion is already in the pool of terms of
 * a vocabulary.
 *
 * @param string $name
 *   The annotation suggestion.
 * @param int $vid
 *   The vocabulary id.
 *
 * @return $tid
 *   The tid if the suggestion is known or NULL.
 */
function _robotagger_check_term_is_new($name, $vid) {
  $tid = NULL;
  $terms = taxonomy_get_term_by_name($name);
  foreach ($terms as $term) {
    if ($term->vid == $vid) {
      $tid = $term->tid;
      break;
    }
  }
  return $tid;
}

/**
 * Prepares the data for _robotagger_process_node_suggestions().
 *
 * @param stdClass $node
 *   The node object.
 *
 * @see robotagger_node_page_form_submit().
 *   The property robotagger_suggestions of the node object is set by
 *   this submit handler.
 */
function _robotagger_process_node_form($node) {
  if (!empty($node->robotagger_suggestions)) {
    $vids = $newsuggestions = $oldtids = $cleanupsuggestions = array();
    $nodeterms = taxonomy_node_get_terms($node);
    foreach ($node->robotagger_suggestions as $vid => $suggestions) {
      $vids[] = $vid;
      foreach ($suggestions as $suggestion) {
        // Does taxonomy known this annotation suggestion
        $tid = _robotagger_check_term_is_new($suggestion, $vid);
        if (empty($tid)) {
          $newsuggestions[$vid][$suggestion] = array('value' => $suggestion);
        }
        elseif (empty($nodeterms[$tid])) {
          $oldtids[$vid][] = array('tid' => $tid);
        }
        elseif (!empty($nodeterms[$tid])) {
          $cleanupsuggestions[$vid][$suggestion] = $suggestion;
        }
      }
    }
    _robotagger_process_node_suggestions($node, $newsuggestions, $oldtids);
    $cleanupsuggestions += $node->robotagger_suggestions;
    _robotagger_suggestions_delete($node, $cleanupsuggestions);
  }
  return;
}

/**
 * Saves the suggestions in the robotagger db table, so that the user can confirm the suggestions.
 *
 * @param stdClass $node
 *   The node object.
 * @param int $vid
 *   The vocabulary id.
 * @param array $newsuggestions
 *   An array of new suggestions.
 */
function _robotagger_suggestions_save($node, $newsuggestions) {
  if (isset($node->nid)) {
    $new = FALSE;
    $saved_suggestions = _robotagger_get_suggestions($node);
    foreach ($newsuggestions as $vid => $items) {
      foreach ($items as $item) {
        if (isset($saved_suggestions[$vid][$item['value']])) {
          continue;
        }
        db_query("INSERT INTO {robotagger_suggestions} (nid, nvid, vid, annotation) VALUES( %d, %d, %d, '%s')",
          $node->nid, $node->vid, $vid, $item['value']
        );
        $new = TRUE;
      }
    }
    //Do not show on our confirmation page.
    if ($new && arg(2) != 'robotagger') {
      $link = l(t('new suggestions'), 'node/' . $node->nid . '/robotagger');
      drupal_set_message(t('The RoboTagger web service was requested. Please confirm the returned !new_tags', array('!new_tags' => $link)), 'status', FALSE);
    }
  }
}

/**
 * Deletes suggestions.
 *
 * @param stdClass $node
 *   The node object.
 * @param array $suggestions
 *   An array of suggestions.
 */
function _robotagger_suggestions_delete($node, $suggestions) {
  if (!empty($suggestions) && !empty($node->nid)) {
    foreach ($suggestions as $vid => $names) {
      foreach ($names as $name) {
        db_query("DELETE FROM {robotagger_suggestions} WHERE nid = %d AND nvid = %d AND vid = %d AND annotation = '%s'", $node->nid, $node->vid, $vid, $name);
      }
    }
  }
}

/**
* Flags suggestions as declined.
*
* @param stdClass $node
*   The node object.
*/
function _robotagger_suggestions_decline($node) {
  if (!empty($node->robotagger_suggestions) && !empty($node->nid)) {
    foreach ($node->robotagger_suggestions as $vid => $names) {
      foreach ($names as $name) {
        db_query("UPDATE {robotagger_suggestions} SET declined = 1 WHERE nid = %d AND nvid = %d AND vid = %d AND annotation = '%s'", $node->nid, $node->vid, $vid, $name);
      }
    }
  }
}


